@@ -13,7 +13,6 @@ import numpy as np
 
 from ginga.util import wcs, wcsmod
 from ginga.BaseImage import BaseImage, ImageError, Header
-from ginga.misc import Bunch
 
 
 class AstroHeader(Header):
@@ -91,6 +90,7 @@ class AstroImage(BaseImage):
 
         # this is a handle to the full data array
         self._md_data = data
+        self.axisdim = data.shape
 
         # this will get reset in set_naxispath() if array is
         # multidimensional
@@ -108,6 +108,14 @@ class AstroImage(BaseImage):
     def load_hdu(self, hdu, fobj=None, naxispath=None,
                  inherit_primary_header=None):
 
+        # this seems to be necessary now for some fits files...
+        try:
+            hdu.verify('fix')
+
+        except Exception as e:
+            # Let's hope for the best!
+            self.logger.warning("Problem verifying fits HDU: {}".format(e))
+
         self.clear_metadata()
 
         # collect HDU header
@@ -480,14 +488,7 @@ class AstroImage(BaseImage):
     # <----- TODO: deprecate
 
     def info_xy(self, data_x, data_y, settings):
-        # Get the value under the data coordinates
-        try:
-            # We report the value across the pixel, even though the coords
-            # change halfway across the pixel
-            value = self.get_data_xy(int(data_x + 0.5), int(data_y + 0.5))
-
-        except Exception as e:
-            value = None
+        info = super(AstroImage, self).info_xy(data_x, data_y, settings)
 
         system = settings.get('wcs_coords', None)
         format = settings.get('wcs_display', 'sexagesimal')
@@ -558,11 +559,8 @@ class AstroImage(BaseImage):
                 tb_str = "Traceback information unavailable."
                 self.logger.error(tb_str)
 
-        info = Bunch.Bunch(itype='astro', data_x=data_x, data_y=data_y,
-                           x=data_x, y=data_y,
-                           ra_txt=ra_txt, dec_txt=dec_txt,
-                           ra_lbl=ra_lbl, dec_lbl=dec_lbl,
-                           value=value)
+        info.update(dict(itype='astro', ra_txt=ra_txt, dec_txt=dec_txt,
+                         ra_lbl=ra_lbl, dec_lbl=dec_lbl))
         return info
 
 # END
