@@ -33,6 +33,9 @@ class CompoundMixin(object):
         self.opaque = False
         self._contains_reduce = np.logical_or
 
+    def __contains__(self, key):
+        return key in self.objects
+
     def get_llur(self):
         """
         Get lower-left and upper-right coordinates of the bounding box
@@ -63,7 +66,10 @@ class CompoundMixin(object):
     def get_items_at(self, pt):
         res = []
         for obj in self.objects:
-            if obj.contains_pt(pt):
+            if obj.is_compound() and not obj.opaque:
+                # non-opaque compound object, list up compatible members
+                res.extend(obj.get_items_at(pt))
+            elif obj.contains_pt(pt):
                 #res.insert(0, obj)
                 res.append(obj)
         return res
@@ -234,12 +240,6 @@ class CompoundMixin(object):
 
     get_center_pt = get_reference_pt
 
-    def reorder_layers(self):
-        self.objects.sort(key=lambda obj: getattr(obj, '_zorder', 0))
-        for obj in self.objects:
-            if obj.is_compound():
-                obj.reorder_layers()
-
     def get_points(self):
         res = []
         for obj in self.objects:
