@@ -12,7 +12,7 @@ from ginga.misc import Bunch
 __all__ = ['TransformError', 'BaseTransform', 'ComposedTransform',
            'InvertedTransform', 'PassThruTransform',
            'WindowNativeTransform', 'CartesianWindowTransform',
-           'CartesianNativeTransform',
+           'CartesianNativeTransform', 'AsIntegerTransform',
            'RotationTransform', 'ScaleTransform',
            'DataCartesianTransform', 'OffsetDataTransform',
            'WCSDataTransform', 'get_catalog'
@@ -37,23 +37,8 @@ class BaseTransform(object):
     def __add__(self, trans):
         return ComposedTransform(self, trans)
 
-    @classmethod
-    def inverted_class(cls):
-        class _inverted(cls):
-
-            def __init__(self, *args, **kwargs):
-                cls.__init__(self, *args, **kwargs)
-
-            def to_(self, pts, **kwargs):
-                return cls.from_(self, pts, **kwargs)
-
-            def from_(self, pts, **kwargs):
-                return cls.to_(self, pts, **kwargs)
-
-        return _inverted
-
     def invert(self):
-        return self.inverted_class()(self)
+        return InvertedTransform(self)
 
 
 class ComposedTransform(BaseTransform):
@@ -272,7 +257,6 @@ class CartesianNativeTransform(BaseTransform):
 
         win_pts = np.add(np.multiply(off_pts, mpy_pt), ctr_pt)
 
-        # round to pixel units, if asked
         # round to pixel units, if asked
         if self.as_int:
             win_pts = np.rint(win_pts).astype(np.int, copy=False)
@@ -305,6 +289,87 @@ class CartesianNativeTransform(BaseTransform):
         return off_pts
 
 
+class AsIntegerTransform(BaseTransform):
+    """
+    A transform from floating point coordinates to integer coordinates.
+    """
+
+    def __init__(self, viewer):
+        super(AsIntegerTransform, self).__init__()
+        self.viewer = viewer
+
+    def to_(self, flt_pts):
+        int_pts = np.asarray(flt_pts, dtype=np.int)
+        return int_pts
+
+    def from_(self, int_pts):
+        """Reverse of :meth:`to_`."""
+        flt_pts = np.asarray(int_pts, dtype=np.float)
+        return flt_pts
+
+
+class FlipSwapTransform(BaseTransform):
+    """
+    A transform in cartesian coordinates based on the flip/swap setting
+    of a viewer.
+    """
+
+    def __init__(self, viewer):
+        super(FlipSwapTransform, self).__init__()
+        self.viewer = viewer
+
+    def to_(self, off_pts):
+        off_pts = np.asarray(off_pts, dtype=np.float)
+        has_z = (off_pts.shape[-1] > 2)
+
+        t_ = self.viewer.t_
+
+        # flip
+        flip_pt = [1.0, 1.0]
+        if t_['flip_x']:
+            flip_pt[0] = -1.0
+        if t_['flip_y']:
+            flip_pt[1] = -1.0
+        if has_z:
+            # no flip_z at the moment
+            flip_pt.append(1.0)
+
+        off_pts = np.multiply(off_pts, flip_pt)
+
+        # swap
+        if t_['swap_xy']:
+            p = list(off_pts.T)
+            off_pts = np.asarray([p[1], p[0]] + list(p[2:])).T
+
+        return off_pts
+
+    def from_(self, off_pts):
+        """Reverse of :meth:`to_`."""
+        off_pts = np.asarray(off_pts, dtype=np.float)
+        has_z = (off_pts.shape[-1] > 2)
+
+        t_ = self.viewer.t_
+
+        # swap
+        if t_['swap_xy']:
+            p = list(off_pts.T)
+            off_pts = np.asarray([p[1], p[0]] + list(p[2:])).T
+
+        # flip
+        flip_pt = [1.0, 1.0]
+        if t_['flip_x']:
+            flip_pt[0] = -1.0
+        if t_['flip_y']:
+            flip_pt[1] = -1.0
+        if has_z:
+            # no flip_z at the moment
+            flip_pt.append(1.0)
+
+        off_pts = np.multiply(off_pts, flip_pt)
+
+        return off_pts
+
+
 class RotationTransform(BaseTransform):
     """
     A transform in cartesian coordinates based on the flip/swap setting and
@@ -315,6 +380,50 @@ class RotationTransform(BaseTransform):
         super(RotationTransform, self).__init__()
         self.viewer = viewer
 
+    def to_(self, off_pts):
+        off_pts = np.asarray(off_pts, dtype=np.float)
+        has_z = (off_pts.shape[-1] > 2)
+
+        t_ = self.viewer.t_
+
+        # rotate
+        if t_['rot_deg'] != 0:
+            thetas = [t_['rot_deg']]
+            offset = [0.0, 0.0]
+            if has_z:
+                offset.append(0.0)
+            off_pts = trcalc.rotate_coord(off_pts, thetas, offset)
+
+        return off_pts
+
+    def from_(self, off_pts):
+        """Reverse of :meth:`to_`."""
+        off_pts = np.asarray(off_pts, dtype=np.float)
+        has_z = (off_pts.shape[-1] > 2)
+
+        t_ = self.viewer.t_
+
+        # rotate
+        if t_['rot_deg'] != 0:
+            thetas = [- t_['rot_deg']]
+            offset = [0.0, 0.0]
+            if has_z:
+                offset.append(0.0)
+            off_pts = trcalc.rotate_coord(off_pts, thetas, offset)
+
+        return off_pts
+
+
+class RotationFlipTransform(BaseTransform):
+    """
+    A transform in cartesian coordinates based on the flip/swap setting and
+    rotation setting of a viewer.
+    """
+
+    def __init__(self, viewer):
+        super(RotationFlipTransform, self).__init__()
+        self.viewer = viewer
+
     def to_(self, off_pts):
         off_pts = np.asarray(off_pts, dtype=np.float)
         has_z = (off_pts.shape[-1] > 2)
@@ -398,9 +507,10 @@ class ScaleTransform(BaseTransform):
         has_z = (off_pts.shape[-1] > 2)
 
         # scale according to current settings
-        scale_pt = [self.viewer._org_scale_x, self.viewer._org_scale_y]
+        sc = self.viewer.renderer.get_scale()
+        scale_pt = [sc[0], sc[1]]
         if has_z:
-            scale_pt.append(self.viewer._org_scale_z)
+            scale_pt.append(sc[2])
 
         off_pts = np.multiply(off_pts, scale_pt)
         return off_pts
@@ -409,10 +519,10 @@ class ScaleTransform(BaseTransform):
         off_pts = np.asarray(off_pts, dtype=np.float)
         has_z = (off_pts.shape[-1] > 2)
 
-        scale_pt = [1.0 / self.viewer._org_scale_x,
-                    1.0 / self.viewer._org_scale_y]
+        sc = self.viewer.renderer.get_scale()
+        scale_pt = [1.0 / sc[0], 1.0 / sc[1]]
         if has_z:
-            scale_pt.append(1.0 / self.viewer._org_scale_z)
+            scale_pt.append(1.0 / sc[2])
 
         # Reverse scaling
         off_pts = np.multiply(off_pts, scale_pt)
@@ -443,9 +553,10 @@ class DataCartesianTransform(BaseTransform):
             data_pts = data_pts - self.viewer.data_off
 
         # subtract data indexes at center reference pixel
-        ref_pt = [self.viewer._org_x, self.viewer._org_y]
+        origin = self.viewer.renderer.get_origin()
+        ref_pt = [origin[0], origin[1]]
         if has_z:
-            ref_pt.append(self.viewer._org_z)
+            ref_pt.append(origin[2])
 
         off_pts = np.subtract(data_pts, ref_pt)
         return off_pts
@@ -456,9 +567,10 @@ class DataCartesianTransform(BaseTransform):
 
         # Add data index at center to offset
         # subtract data indexes at center reference pixel
-        ref_pt = [self.viewer._org_x, self.viewer._org_y]
+        origin = self.viewer.renderer.get_origin()
+        ref_pt = [origin[0], origin[1]]
         if has_z:
-            ref_pt.append(self.viewer._org_z)
+            ref_pt.append(origin[2])
 
         data_pts = np.add(off_pts, ref_pt)
 
