@@ -9,7 +9,6 @@
 import math
 import numpy as np
 
-from ginga.AstroImage import AstroImage
 from ginga.canvas.CanvasObject import (CanvasObjectBase, _bool, _color,
                                        Point, MovePoint, ScalePoint,
                                        register_canvas_types, get_canvas_type,
@@ -27,7 +26,7 @@ __all__ = ['Ruler', 'Compass', 'Crosshair', 'AnnulusMixin', 'Annulus',
            'Annulus2R', 'WCSAxes']
 
 
-class Ruler(TwoPointMixin, CanvasObjectBase):
+class RulerP(TwoPointMixin, CanvasObjectBase):
     """
     Draws a WCS ruler (like a right triangle) on a DrawingCanvas.
     Parameters are:
@@ -86,14 +85,14 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
 
     @classmethod
     def idraw(cls, canvas, cxt):
-        return cls(cxt.start_x, cxt.start_y, cxt.x, cxt.y, **cxt.drawparams)
+        return cls((cxt.start_x, cxt.start_y), (cxt.x, cxt.y), **cxt.drawparams)
 
-    def __init__(self, x1, y1, x2, y2, color='green', color2='skyblue',
+    def __init__(self, pt1, pt2, color='green', color2='skyblue',
                  alpha=1.0, linewidth=1, linestyle='solid',
                  showcap=True, showplumb=True, showends=False, units='arcmin',
                  font='Sans Serif', fontsize=None, **kwdargs):
         self.kind = 'ruler'
-        points = np.asarray([(x1, y1), (x2, y2)], dtype=np.float)
+        points = np.asarray([pt1, pt2], dtype=np.float)
         CanvasObjectBase.__init__(self, color=color, color2=color2,
                                   alpha=alpha, units=units,
                                   showplumb=showplumb, showends=showends,
@@ -247,7 +246,25 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
             self.draw_caps(cr, self.cap, ((cx2, cy1), ))
 
 
-class Compass(OnePointOneRadiusMixin, CanvasObjectBase):
+class Ruler(RulerP):
+
+    @classmethod
+    def idraw(cls, canvas, cxt):
+        return cls(cxt.start_x, cxt.start_y, cxt.x, cxt.y, **cxt.drawparams)
+
+    def __init__(self, x1, y1, x2, y2, color='green', color2='skyblue',
+                 alpha=1.0, linewidth=1, linestyle='solid',
+                 showcap=True, showplumb=True, showends=False, units='arcmin',
+                 font='Sans Serif', fontsize=None, **kwdargs):
+        RulerP.__init__(self, (x1, y1), (x2, y2), color=color, color2=color2,
+                        alpha=alpha, units=units,
+                        showplumb=showplumb, showends=showends,
+                        linewidth=linewidth, showcap=showcap,
+                        linestyle=linestyle, font=font, fontsize=fontsize,
+                        **kwdargs)
+
+
+class CompassP(OnePointOneRadiusMixin, CanvasObjectBase):
     """
     Draws a WCS compass on a DrawingCanvas.
     Parameters are:
@@ -298,13 +315,13 @@ class Compass(OnePointOneRadiusMixin, CanvasObjectBase):
     def idraw(cls, canvas, cxt):
         radius = np.sqrt(abs(cxt.start_x - cxt.x) ** 2 +
                          abs(cxt.start_y - cxt.y) ** 2)
-        return cls(cxt.start_x, cxt.start_y, radius, **cxt.drawparams)
+        return cls((cxt.start_x, cxt.start_y), radius, **cxt.drawparams)
 
-    def __init__(self, x, y, radius, ctype='wcs', color='skyblue',
+    def __init__(self, pt, radius, ctype='wcs', color='skyblue',
                  linewidth=1, fontsize=None, font='Sans Serif',
                  alpha=1.0, linestyle='solid', showcap=True, **kwdargs):
         self.kind = 'compass'
-        points = np.asarray([(x, y)], dtype=np.float)
+        points = np.asarray([pt], dtype=np.float)
         CanvasObjectBase.__init__(self, ctype=ctype, color=color, alpha=alpha,
                                   linewidth=linewidth, showcap=showcap,
                                   linestyle=linestyle,
@@ -440,7 +457,24 @@ class Compass(OnePointOneRadiusMixin, CanvasObjectBase):
         return (xd, yd)
 
 
-class Crosshair(OnePointMixin, CanvasObjectBase):
+class Compass(CompassP):
+
+    @classmethod
+    def idraw(cls, canvas, cxt):
+        radius = np.sqrt(abs(cxt.start_x - cxt.x) ** 2 +
+                         abs(cxt.start_y - cxt.y) ** 2)
+        return cls(cxt.start_x, cxt.start_y, radius, **cxt.drawparams)
+
+    def __init__(self, x, y, radius, ctype='wcs', color='skyblue',
+                 linewidth=1, fontsize=None, font='Sans Serif',
+                 alpha=1.0, linestyle='solid', showcap=True, **kwdargs):
+        CompassP.__init__(self, (x, y), radius, ctype=ctype, color=color,
+                          alpha=alpha, linewidth=linewidth, showcap=showcap,
+                          linestyle=linestyle, font=font, fontsize=fontsize,
+                          **kwdargs)
+
+
+class CrosshairP(OnePointMixin, CanvasObjectBase):
     """
     Draws a crosshair on a DrawingCanvas.
     Parameters are:
@@ -490,15 +524,15 @@ class Crosshair(OnePointMixin, CanvasObjectBase):
 
     @classmethod
     def idraw(cls, canvas, cxt):
-        return cls(cxt.x, cxt.y, **cxt.drawparams)
+        return cls((cxt.x, cxt.y), **cxt.drawparams)
 
-    def __init__(self, x, y, color='green',
+    def __init__(self, pt, color='green',
                  linewidth=1, alpha=1.0, linestyle='solid',
                  text=None, textcolor='yellow',
                  fontsize=10.0, font='Sans Serif', fontscale=True,
                  format='xy', **kwdargs):
         self.kind = 'crosshair'
-        points = np.asarray([(x, y)], dtype=np.float)
+        points = np.asarray([pt], dtype=np.float)
         CanvasObjectBase.__init__(self, color=color, alpha=alpha,
                                   linewidth=linewidth, linestyle=linestyle,
                                   text=text, textcolor=textcolor,
@@ -529,9 +563,7 @@ class Crosshair(OnePointMixin, CanvasObjectBase):
                 text = "X:%f, Y:%f" % (self.x, self.y)
 
             else:
-                image = viewer.get_image()
-                if image is None:
-                    return
+                image = viewer.get_vip()
                 # NOTE: x, y are assumed to be in data coordinates
                 info = image.info_xy(self.x, self.y, viewer.get_settings())
                 if self.format == 'coords':
@@ -541,7 +573,9 @@ class Crosshair(OnePointMixin, CanvasObjectBase):
                         text = "%s:%s, %s:%s" % (info.ra_lbl, info.ra_txt,
                                                  info.dec_lbl, info.dec_txt)
                 else:
-                    if np.isscalar(info.value) or len(info.value) <= 1:
+                    if info.value is None:
+                        text = "V: None"
+                    elif np.isscalar(info.value) or len(info.value) <= 1:
                         text = "V: %f" % (info.value)
                     else:
                         values = ', '.join(["%d" % info.value[i]
@@ -564,6 +598,25 @@ class Crosshair(OnePointMixin, CanvasObjectBase):
         cr.draw_text(cx + 10, cy + 4 + txtht, text)
 
 
+class Crosshair(CrosshairP):
+
+    @classmethod
+    def idraw(cls, canvas, cxt):
+        return cls(cxt.x, cxt.y, **cxt.drawparams)
+
+    def __init__(self, x, y, color='green',
+                 linewidth=1, alpha=1.0, linestyle='solid',
+                 text=None, textcolor='yellow',
+                 fontsize=10.0, font='Sans Serif', fontscale=True,
+                 format='xy', **kwdargs):
+        CrosshairP.__init__(self, (x, y), color=color, alpha=alpha,
+                            linewidth=linewidth, linestyle=linestyle,
+                            text=text, textcolor=textcolor,
+                            fontsize=fontsize, font=font,
+                            fontscale=fontscale,
+                            format=format, **kwdargs)
+
+
 class AnnulusMixin(object):
 
     def contains_pt(self, pt):
@@ -588,7 +641,7 @@ class AnnulusMixin(object):
         return obj2.select_contains_pt(viewer, pt)
 
 
-class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
+class AnnulusP(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
     """
     Special compound object to handle annulus shape that
     consists of two objects with the same centroid.
@@ -637,14 +690,13 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
     def idraw(cls, canvas, cxt):
         radius = np.sqrt(abs(cxt.start_x - cxt.x)**2 +
                          abs(cxt.start_y - cxt.y)**2)
-        return cls(cxt.start_x, cxt.start_y, radius,
+        return cls((cxt.start_x, cxt.start_y), radius,
                    **cxt.drawparams)
 
-    def __init__(self, x, y, radius, width=None,
+    def __init__(self, pt, radius, width=None,
                  atype='circle', color='yellow',
                  linewidth=1, linestyle='solid', alpha=1.0,
                  **kwdargs):
-
         if width is None:
             # default width is 15% of radius
             width = 0.15 * radius
@@ -656,19 +708,19 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
         coord = kwdargs.get('coord', None)
 
         klass = get_canvas_type(atype)
-        obj1 = klass(x, y, radius, color=color,
+        obj1 = klass(pt[0], pt[1], radius, color=color,
                      linewidth=linewidth,
                      linestyle=linestyle, alpha=alpha,
                      coord=coord)
         obj1.editable = False
 
-        obj2 = klass(x, y, oradius, color=color,
+        obj2 = klass(pt[0], pt[1], oradius, color=color,
                      linewidth=linewidth,
                      linestyle=linestyle, alpha=alpha,
                      coord=coord)
         obj2.editable = False
 
-        points = np.asarray([(x, y)], dtype=np.float)
+        points = np.asarray([pt], dtype=np.float)
 
         CompoundObject.__init__(self, obj1, obj2,
                                 points=points, radius=radius,
@@ -738,12 +790,30 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
         self.objects[1].__dict__.update(d)
 
     def move_to_pt(self, dst_pt):
-        super(Annulus, self).move_to_pt(dst_pt)
+        super(AnnulusP, self).move_to_pt(dst_pt)
 
         self.set_data_points([dst_pt])
 
 
-class Annulus2R(AnnulusMixin, OnePointTwoRadiusMixin, CompoundObject):
+class Annulus(AnnulusP):
+
+    @classmethod
+    def idraw(cls, canvas, cxt):
+        radius = np.sqrt(abs(cxt.start_x - cxt.x)**2 +
+                         abs(cxt.start_y - cxt.y)**2)
+        return cls(cxt.start_x, cxt.start_y, radius,
+                   **cxt.drawparams)
+
+    def __init__(self, x, y, radius, width=None,
+                 atype='circle', color='yellow',
+                 linewidth=1, linestyle='solid', alpha=1.0,
+                 **kwdargs):
+        AnnulusP.__init__(self, (x, y), radius, width=width, atype=atype,
+                          color=color, linewidth=linewidth,
+                          linestyle=linestyle, alpha=alpha, **kwdargs)
+
+
+class Annulus2RP(AnnulusMixin, OnePointTwoRadiusMixin, CompoundObject):
     """
     Special compound object to handle annulus shape that
     consists of two objects, (one center point, plus two radii).
@@ -800,13 +870,14 @@ class Annulus2R(AnnulusMixin, OnePointTwoRadiusMixin, CompoundObject):
     @classmethod
     def idraw(cls, canvas, cxt):
         xradius, yradius = abs(cxt.start_x - cxt.x), abs(cxt.start_y - cxt.y)
-        return cls(cxt.start_x, cxt.start_y, xradius, yradius, **cxt.drawparams)
+        return cls((cxt.start_x, cxt.start_y), (xradius, yradius),
+                   **cxt.drawparams)
 
-    def __init__(self, x, y, xradius, yradius, xwidth=None,
+    def __init__(self, pt, radii, xwidth=None,
                  ywidth=None, atype='ellipse', color='yellow',
                  linewidth=1, linestyle='solid', alpha=1.0,
                  rot_deg=0.0, **kwdargs):
-
+        xradius, yradius = radii
         if xwidth is None:
             # default X width is 15% of X radius
             xwidth = 0.15 * xradius
@@ -822,19 +893,19 @@ class Annulus2R(AnnulusMixin, OnePointTwoRadiusMixin, CompoundObject):
         coord = kwdargs.get('coord', None)
 
         klass = get_canvas_type(atype)
-        obj1 = klass(x, y, xradius, yradius, color=color,
+        obj1 = klass(pt[0], pt[1], radii[0], radii[1], color=color,
                      linewidth=linewidth,
                      linestyle=linestyle, alpha=alpha,
                      coord=coord, rot_deg=rot_deg)
         obj1.editable = False
 
-        obj2 = klass(x, y, oxradius, oyradius, color=color,
+        obj2 = klass(pt[0], pt[1], oxradius, oyradius, color=color,
                      linewidth=linewidth,
                      linestyle=linestyle, alpha=alpha,
                      coord=coord, rot_deg=rot_deg)
         obj2.editable = False
 
-        points = np.asarray([(x, y)], dtype=np.float)
+        points = np.asarray([pt], dtype=np.float)
 
         CompoundObject.__init__(self, obj1, obj2,
                                 points=points, xradius=xradius, yradius=yradius,
@@ -943,11 +1014,30 @@ class Annulus2R(AnnulusMixin, OnePointTwoRadiusMixin, CompoundObject):
         self.objects[1].__dict__.update(d)
 
     def move_to_pt(self, dst_pt):
-        super(Annulus2R, self).move_to_pt(dst_pt)
+        super(Annulus2RP, self).move_to_pt(dst_pt)
 
         self.set_data_points([dst_pt])
 
 
+class Annulus2R(Annulus2RP):
+
+    @classmethod
+    def idraw(cls, canvas, cxt):
+        xradius, yradius = abs(cxt.start_x - cxt.x), abs(cxt.start_y - cxt.y)
+        return cls(cxt.start_x, cxt.start_y, xradius, yradius,
+                   **cxt.drawparams)
+
+    def __init__(self, x, y, xradius, yradius, xwidth=None,
+                 ywidth=None, atype='ellipse', color='yellow',
+                 linewidth=1, linestyle='solid', alpha=1.0,
+                 rot_deg=0.0, **kwdargs):
+        Annulus2RP.__init__(self, (x, y), (xradius, yradius),
+                            xwidth=xwidth, ywidth=ywidth, atype=atype,
+                            color=color, linewidth=linewidth,
+                            linestyle=linestyle, alpha=alpha,
+                            rot_deg=rot_deg, **kwdargs)
+
+
 class WCSAxes(CompoundObject):
     """
     Special compound object to draw WCS axes.
@@ -990,7 +1080,8 @@ class WCSAxes(CompoundObject):
         # for keeping track of changes to image and orientation
         self._cur_rot = None
         self._cur_swap = None
-        self._cur_image = None
+        self._cur_limits = ((0.0, 0.0), (0.0, 0.0))
+        self._cur_images = set([])
 
         CompoundObject.__init__(self,
                                 color=color, alpha=alpha,
@@ -1002,27 +1093,30 @@ class WCSAxes(CompoundObject):
         self.opaque = True
         self.kind = 'wcsaxes'
 
-    def _calc_axes(self, viewer, image, rot_deg, swapxy):
-        self._cur_image = image
+    def _calc_axes(self, viewer, images, rot_deg, swapxy, limits):
+        self.logger.debug("recalculating axes...")
+        self._cur_images = images
         self._cur_rot = rot_deg
         self._cur_swap = swapxy
+        self._cur_limits = limits
 
-        if not isinstance(image, AstroImage) or not image.has_valid_wcs():
+        image = viewer.get_image()
+        if image is None or not image.has_valid_wcs():
             self.logger.debug(
-                'WCSAxes can only be displayed for AstroImage with valid WCS')
+                'WCSAxes can only be displayed for image with valid WCS')
             return []
 
-        min_imsize = min(image.width, image.height)
+        x1, y1 = limits[0][:2]
+        x2, y2 = limits[1][:2]
+        min_imsize = min(x2 - x1, y2 - y1)
         if min_imsize <= 0:
             self.logger.debug('Cannot draw WCSAxes on image with 0 dim')
             return []
 
         # Approximate bounding box in RA/DEC space
-        xmax = image.width - 1
-        ymax = image.height - 1
         try:
             radec = image.wcs.datapt_to_system(
-                [[0, 0], [0, ymax], [xmax, 0], [xmax, ymax]],
+                [(x1, y1), (x1, y2), (x2, y1), (x2, y2)],
                 naxispath=image.naxispath)
         except Exception as e:
             self.logger.warning('WCSAxes failed: {}'.format(str(e)))
@@ -1067,9 +1161,10 @@ class WCSAxes(CompoundObject):
             self.logger.warning('WCSAxes failed: {}'.format(str(e)))
             return []
 
+        (x1, y1), (x2, y2) = viewer.get_limits()
         # Don't draw outside image area
-        mask = ((pts[:, 0] >= 0) & (pts[:, 0] < image.width) &
-                (pts[:, 1] >= 0) & (pts[:, 1] < image.height))
+        mask = ((pts[:, 0] >= x1) & (pts[:, 0] <= x2) &
+                (pts[:, 1] >= y1) & (pts[:, 1] <= y2))
         pts = pts[mask]
 
         if len(pts) == 0:
@@ -1160,11 +1255,12 @@ class WCSAxes(CompoundObject):
 
     def draw(self, viewer):
         # see if we need to recalculate our grid
-        image = viewer.get_image()
-        update = False
-        if self._cur_image != image:
-            # new image loaded
-            update = True
+        canvas = viewer.get_canvas()
+        vip = viewer.get_vip()
+        cvs_imgs = vip.get_images([], canvas)
+        images = set([cvs_img.get_image() for cvs_img in cvs_imgs])
+        diff = images.difference(self._cur_images)
+        update = len(diff) > 0
 
         cur_swap = viewer.get_transforms()[2]
         if cur_swap != self._cur_swap:
@@ -1179,6 +1275,11 @@ class WCSAxes(CompoundObject):
             # and update all the text objects in self.objects
             update = True
 
+        cur_limits = viewer.get_limits()
+        if not np.all(np.isclose(cur_limits, self._cur_limits)):
+            # limits have changed
+            update = True
+
         if len(self.objects) == 0:
             # initial time
             update = True
@@ -1187,7 +1288,8 @@ class WCSAxes(CompoundObject):
             # only expensive recalculation of grid if needed
             self.ra_angle = None
             self.dec_angle = None
-            self.objects = self._calc_axes(viewer, image, cur_rot, cur_swap)
+            self.objects = self._calc_axes(viewer, images, cur_rot, cur_swap,
+                                           cur_limits)
 
         super(WCSAxes, self).draw(viewer)
 
