@@ -74,12 +74,8 @@ class CvContext(object):
         self.canvas = canvas
 
     def get_color(self, color, alpha=1.0):
-        if isinstance(color, str) or isinstance(color, type(u"")):
-            r, g, b = colors.lookup_color(color)
-        elif isinstance(color, tuple):
-            # color is assumed to be a 3-tuple of RGB values as floats
-            # between 0 and 1
-            r, g, b = color
+        if color is not None:
+            r, g, b = colors.resolve_color(color)
         else:
             r, g, b = 1.0, 1.0, 1.0
 
@@ -112,6 +108,13 @@ class CvContext(object):
         wd, ht = retval
         return wd, ht
 
+    def image(self, pt, rgb_arr):
+        # TODO: is there a faster way to copy this array in?
+        cx, cy = pt[:2]
+        daht, dawd, depth = rgb_arr.shape
+
+        self.canvas[cy:cy + daht, cx:cx + dawd, :] = rgb_arr
+
     def text(self, pt, text, font):
         x, y = pt
         font.font.putText(self.canvas, text, (x, y), font.scale,
