@@ -9,7 +9,7 @@
 
 from ginga.gtk3w import GtkHelp
 
-from ginga.misc import Callback, Bunch, LineHistory
+from ginga.misc import Callback, Bunch, Settings, LineHistory
 from functools import reduce
 
 from gi.repository import Gtk
@@ -32,7 +32,7 @@ except Exception:
 
 __all__ = ['WidgetError', 'WidgetBase', 'TextEntry', 'TextEntrySet',
            'TextArea', 'Label', 'Button', 'ComboBox',
-           'SpinBox', 'Slider', 'ScrollBar', 'CheckBox', 'ToggleButton',
+           'SpinBox', 'Slider', 'Dial', 'ScrollBar', 'CheckBox', 'ToggleButton',
            'RadioButton', 'Image', 'ProgressBar', 'StatusBar', 'TreeView',
            'WebView', 'ContainerBase', 'Box', 'HBox', 'VBox', 'Frame',
            'Expander', 'TabWidget', 'StackWidget', 'MDIWidget', 'ScrollArea',
@@ -103,6 +103,7 @@ class WidgetBase(Callback.Callbacks):
 
     def delete(self):
         self.widget.destroy()
+        self.widget = None
 
     def show(self):
         # self.widget.show()
@@ -571,6 +572,39 @@ class Slider(WidgetBase):
         adj.configure(minval, minval, maxval, incr_value, incr_value, 0)
 
 
+class Dial(WidgetBase):
+    def __init__(self, dtype=float, wrap=False, track=False):
+        super(Dial, self).__init__()
+
+        w = GtkHelp.ValueDial()
+        self.widget = w
+
+        w.draw_value = False
+        w.wrap = wrap
+        w.set_tracking(track)
+        w.connect('value-changed', self._cb_redirect)
+        self.dtype = dtype
+
+        self.enable_callback('value-changed')
+
+    def _cb_redirect(self, dial, val):
+        ext_val = self.dtype(val)
+        self.make_callback('value-changed', ext_val)
+
+    def get_value(self):
+        int_val = self.widget.get_value()
+        return self.dtype(int_val)
+
+    def set_value(self, val):
+        self.widget.set_value(val)
+
+    def set_tracking(self, tf):
+        self.widget.set_tracking(tf)
+
+    def set_limits(self, minval, maxval, incr_value=1):
+        self.widget.set_limits(minval, maxval, incr_value)
+
+
 class ScrollBar(WidgetBase):
     def __init__(self, orientation='horizontal'):
         super(ScrollBar, self).__init__()
@@ -1982,13 +2016,17 @@ class TopLevel(TopLevelMixin, ContainerBase):
 
 class Application(Callback.Callbacks):
 
-    def __init__(self, logger=None):
+    def __init__(self, logger=None, settings=None):
         global _app
         super(Application, self).__init__()
 
         self.logger = logger
-        self.window_list = []
+        if settings is None:
+            settings = Settings.SettingGroup(logger=self.logger)
+        self.settings = settings
+        self.settings.add_defaults(font_scaling_factor=None)
 
+        self.window_list = []
         self.window_dict = {}
         self.wincnt = 0
 
@@ -2004,8 +2042,10 @@ class Application(Callback.Callbacks):
 
             self.screen_res = screen.get_resolution()
 
-            # hack for Gtk--scale fonts on HiDPI displays
-            scale = self.screen_res / 72.0
+            scale = self.settings.get('font_scaling_factor', None)
+            if scale is None:
+                # hack for Gtk--scale fonts on HiDPI displays
+                scale = self.screen_res / 72.0
             self.logger.debug("setting default font_scaling_factor={}".format(scale))
             from ginga.fonts import font_asst
             font_asst.default_scaling_factor = scale
