@@ -5,9 +5,6 @@ tkname = toolkit.get_family()
 if tkname == 'qt':
     from ginga.qtw.Plot import *  # noqa
 
-elif tkname == 'gtk':
-    from ginga.gtkw.Plot import *  # noqa
-
 elif tkname == 'gtk3':
     from ginga.gtk3w.Plot import *  # noqa
 
