@@ -4,28 +4,20 @@
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
-from __future__ import absolute_import, print_function
-from ..util import six
-from ..util.six.moves import map
-
 import sys
 import time
 import threading
 import traceback
 
-if six.PY2:
-    import thread
-    import Queue
-else:
-    import _thread as thread
-    import queue as Queue
+import _thread as thread
+import queue as Queue
 
-    # NOTE: See http://bugs.python.org/issue7946
-    # we cannot effectively use threading for loading files/network/etc.
-    # without setting the switchinterval down on python 3 due to the new
-    # GIL implementation
-    _swival = 0.000001
-    sys.setswitchinterval(_swival)
+# NOTE: See http://bugs.python.org/issue7946
+# we cannot effectively use threading for loading files/network/etc.
+# without setting the switchinterval down on python 3 due to the new
+# GIL implementation
+_swival = 0.000001
+sys.setswitchinterval(_swival)
 
 from . import Callback  # noqa
 
