@@ -90,8 +90,15 @@ class RenderContext(render.RenderContextBase):
     def text_extents(self, text):
         return self.cr.text_extents(text, self.font)
 
+    def setup_pen_brush(self, pen, brush):
+        self.pen = pen
+        self.brush = brush
+
     ##### DRAWING OPERATIONS #####
 
+    def draw_image(self, cvs_img, cpoints, rgb_arr, whence, order='RGBA'):
+        return
+
     def draw_text(self, cx, cy, text, rot_deg=0.0):
         fontdict = self.font.get_fontdict()
         self.cr.push(allow=['alpha', 'color'])
@@ -164,10 +171,10 @@ class RenderContext(render.RenderContextBase):
         self.cr.axes.add_patch(p)
 
 
-class CanvasRenderer(render.RendererBase):
+class CanvasRenderer(render.StandardPixelRenderer):
 
     def __init__(self, viewer):
-        render.RendererBase.__init__(self, viewer)
+        render.StandardPixelRenderer.__init__(self, viewer)
 
         self.kind = 'mpl'
         self.rgb_order = viewer.rgb_order
@@ -177,7 +184,7 @@ class CanvasRenderer(render.RendererBase):
         """Resize our drawing area to encompass a space defined by the
         given dimensions.
         """
-        pass
+        super(CanvasRenderer, self).resize(dims)
 
     def render_image(self, rgbobj, dst_x, dst_y):
         # for compatibility with the other renderers
@@ -196,5 +203,11 @@ class CanvasRenderer(render.RendererBase):
         cr.set_font_from_shape(shape)
         return cr.text_extents(shape.text)
 
+    def text_extents(self, text, font):
+        cr = RenderContext(self, self.viewer, self.viewer.ax_util)
+        cr.set_font(font.fontname, font.fontsize, color=font.color,
+                    alpha=font.alpha)
+        return cr.text_extents(text)
+
 
 #END
