@@ -79,6 +79,15 @@ class RenderContext(render.RenderContextBase):
         else:
             self.brush = self.cr.get_brush(color, alpha=alpha)
 
+    def setup_pen_brush(self, pen, brush):
+        # pen, brush are from ginga.vec
+        self.pen = self.cr.get_pen(pen.color, alpha=pen.alpha,
+                                   linewidth=pen.linewidth)
+        if brush is None:
+            self.brush = None
+        else:
+            self.brush = self.cr.get_brush(brush.color, alpha=brush.alpha)
+
     def set_font(self, fontname, fontsize, color='black', alpha=1.0):
         fontsize = self.scale_fontsize(fontsize)
         self.font = self.cr.get_font(fontname, fontsize, color,
@@ -103,6 +112,10 @@ class RenderContext(render.RenderContextBase):
 
     ##### DRAWING OPERATIONS #####
 
+    def draw_image(self, cvs_img, cpoints, rgb_arr, whence, order='RGBA'):
+        # no-op for this renderer
+        pass
+
     def draw_text(self, cx, cy, text, rot_deg=0.0):
         wd, ht = self.cr.text_extents(text, self.font)
 
@@ -124,10 +137,10 @@ class RenderContext(render.RenderContextBase):
         self.cr.path(cpoints, self.pen)
 
 
-class CanvasRenderer(render.RendererBase):
+class CanvasRenderer(render.StandardPixelRenderer):
 
     def __init__(self, viewer):
-        render.RendererBase.__init__(self, viewer)
+        render.StandardPixelRenderer.__init__(self, viewer)
 
         self.kind = 'pil'
         self.rgb_order = 'RGBA'
@@ -139,7 +152,6 @@ class CanvasRenderer(render.RendererBase):
         given dimensions.
         """
         width, height = dims[:2]
-        self.dims = (width, height)
         self.logger.debug("renderer reconfigured to %dx%d" % (
             width, height))
 
@@ -148,6 +160,12 @@ class CanvasRenderer(render.RendererBase):
         # blending, not RGBA
         self.surface = Image.new('RGB', (width, height), color=0)
 
+        super(CanvasRenderer, self).resize(dims)
+
+    ## def finalize(self):
+    ##     cr = RenderContext(self, self.viewer, self.surface)
+    ##     self.draw_vector(cr)
+
     def render_image(self, rgbobj, dst_x, dst_y):
         """Render the image represented by (rgbobj) at dst_x, dst_y
         in the pixel space.
@@ -179,7 +197,7 @@ class CanvasRenderer(render.RendererBase):
         # was last updated?
         wd, ht = self.dims[:2]
 
-        # Get agg surface as a numpy array
+        # Get PIL surface as a numpy array
         arr8 = np.fromstring(self.surface.tobytes(), dtype=np.uint8)
         arr8 = arr8.reshape((ht, wd, 3))
 
@@ -197,4 +215,10 @@ class CanvasRenderer(render.RendererBase):
         cr.set_font_from_shape(shape)
         return cr.text_extents(shape.text)
 
+    def text_extents(self, text, font):
+        cr = RenderContext(self, self.viewer, self.surface)
+        cr.set_font(font.fontname, font.fontsize, color=font.color,
+                    alpha=font.alpha)
+        return cr.text_extents(text)
+
 #END
