@@ -5,17 +5,31 @@
 # Please see the file LICENSE.txt for details.
 #
 import os
-from io import BytesIO
+import tempfile
 
 import numpy as np
 
 from ginga import ImageView, Mixins, Bindings
 from ginga.util.paths import icondir
 from ginga.qtw.QtHelp import (QtGui, QtCore, QImage, QPixmap, QCursor,
-                              QPainter, Timer, get_scroll_info, get_painter)
+                              QPainter, QOpenGLWidget, QSurfaceFormat,
+                              Timer, get_scroll_info, get_painter)
 
 from .CanvasRenderQt import CanvasRenderer
 
+have_opengl = False
+try:
+    from ginga.opengl.CanvasRenderGL import CanvasRenderer as OpenGLRenderer
+    from ginga.opengl.GlHelp import get_transforms
+    from ginga.opengl.glsl import req
+
+    have_opengl = True
+except ImportError:
+    pass
+
+# set to True to debug window painting
+DEBUG_MODE = False
+
 
 class ImageViewQtError(ImageView.ImageViewError):
     pass
@@ -27,13 +41,13 @@ class RenderGraphicsView(QtGui.QGraphicsView):
         super(RenderGraphicsView, self).__init__(*args, **kwdargs)
 
         self.viewer = None
-        self.pixmap = None
 
     def drawBackground(self, painter, rect):
         """When an area of the window is exposed, we just copy out of the
         server-side, off-screen pixmap to that area.
         """
-        if not self.pixmap:
+        pixmap = self.viewer.pixmap
+        if pixmap is None:
             return
         x1, y1, x2, y2 = rect.getCoords()
         width = x2 - x1 + 1
@@ -41,7 +55,7 @@ class RenderGraphicsView(QtGui.QGraphicsView):
 
         # redraw the screen from backing pixmap
         rect = QtCore.QRect(x1, y1, width, height)
-        painter.drawPixmap(rect, self.pixmap, rect)
+        painter.drawPixmap(rect, pixmap, rect)
 
     def resizeEvent(self, event):
         rect = self.geometry()
@@ -57,9 +71,6 @@ class RenderGraphicsView(QtGui.QGraphicsView):
             width, height = self.viewer.get_desired_size()
         return QtCore.QSize(width, height)
 
-    def set_pixmap(self, pixmap):
-        self.pixmap = pixmap
-
 
 class RenderWidget(QtGui.QWidget):
 
@@ -67,14 +78,14 @@ class RenderWidget(QtGui.QWidget):
         super(RenderWidget, self).__init__(*args, **kwdargs)
 
         self.viewer = None
-        self.pixmap = None
         self.setAttribute(QtCore.Qt.WA_OpaquePaintEvent)
 
     def paintEvent(self, event):
         """When an area of the window is exposed, we just copy out of the
         server-side, off-screen pixmap to that area.
         """
-        if not self.pixmap:
+        pixmap = self.viewer.pixmap
+        if pixmap is None:
             return
         rect = event.rect()
         x1, y1, x2, y2 = rect.getCoords()
@@ -83,8 +94,12 @@ class RenderWidget(QtGui.QWidget):
 
         # redraw the screen from backing pixmap
         painter = QPainter(self)
+        #painter = get_painter(self)
         rect = QtCore.QRect(x1, y1, width, height)
-        painter.drawPixmap(rect, self.pixmap, rect)
+        painter.drawPixmap(rect, pixmap, rect)
+        if DEBUG_MODE:
+            qimage = pixmap.toImage()
+            save_debug_image(qimage, 'final_image.png', format='png')
 
     def resizeEvent(self, event):
         rect = self.geometry()
@@ -101,8 +116,35 @@ class RenderWidget(QtGui.QWidget):
             width, height = self.viewer.get_desired_size()
         return QtCore.QSize(width, height)
 
-    def set_pixmap(self, pixmap):
-        self.pixmap = pixmap
+
+class RenderGLWidget(QOpenGLWidget):
+
+    def __init__(self, *args, **kwdargs):
+        QOpenGLWidget.__init__(self, *args, **kwdargs)
+
+        self.viewer = None
+
+        # ensure we are using correct version of opengl
+        fmt = QSurfaceFormat()
+        fmt.setVersion(req.major, req.minor)
+        fmt.setProfile(QSurfaceFormat.CoreProfile)
+        #fmt.setDefaultFormat(fmt)
+        self.setFormat(fmt)
+
+    def initializeGL(self):
+        self.viewer.renderer.gl_initialize()
+
+    def resizeGL(self, width, height):
+        self.viewer.configure_window(width, height)
+
+    def paintGL(self):
+        self.viewer.renderer.gl_paint()
+
+    def sizeHint(self):
+        width, height = 300, 300
+        if self.viewer is not None:
+            width, height = self.viewer.get_desired_size()
+        return QtCore.QSize(width, height)
 
 
 class ImageViewQt(ImageView.ImageViewBase):
@@ -112,13 +154,17 @@ class ImageViewQt(ImageView.ImageViewBase):
                                          rgbmap=rgbmap, settings=settings)
 
         if render is None:
-            render = 'widget'
+            render = self.t_.get('render_widget', 'widget')
         self.wtype = render
         if self.wtype == 'widget':
             self.imgwin = RenderWidget()
         elif self.wtype == 'scene':
             self.scene = QtGui.QGraphicsScene()
             self.imgwin = RenderGraphicsView(self.scene)
+        elif self.wtype == 'opengl':
+            if not have_opengl:
+                raise ImageViewQtError("Please install 'pyopengl' to use render: '%s'" % (render))
+            self.imgwin = RenderGLWidget()
         else:
             raise ImageViewQtError("Undefined render type: '%s'" % (render))
         self.imgwin.viewer = self
@@ -126,11 +172,19 @@ class ImageViewQt(ImageView.ImageViewBase):
         self.qimg_fmt = QImage.Format_RGB32
         # find out optimum format for backing store
         #self.qimg_fmt = QPixmap(1, 1).toImage().format()
-        # Qt needs this to be in BGR(A)
-        self.rgb_order = 'BGRA'
 
-        # default renderer is Qt one
-        self.renderer = CanvasRenderer(self, surface_type='qpixmap')
+        if self.wtype == 'opengl':
+            if not have_opengl:
+                raise ValueError("OpenGL imports failed")
+            self.rgb_order = 'RGBA'
+            self.renderer = OpenGLRenderer(self)
+            # we replace some transforms in the catalog for OpenGL rendering
+            self.tform = get_transforms(self)
+        else:
+            # Qt needs this to be in BGR(A)
+            self.rgb_order = 'BGRA'
+            # default renderer is Qt one
+            self.renderer = CanvasRenderer(self, surface_type='qpixmap')
 
         self.msgtimer = Timer()
         self.msgtimer.add_callback('expired',
@@ -156,13 +210,15 @@ class ImageViewQt(ImageView.ImageViewBase):
             self.scene.setSceneRect(1, 1, width - 2, height - 2)
 
         # tell renderer about our new size
-        self.renderer.resize((width, height))
+        #self.renderer.resize((width, height))
 
-        if isinstance(self.renderer.surface, QPixmap):
+        if self.wtype == 'opengl':
+            pass
+
+        elif isinstance(self.renderer.surface, QPixmap):
             # optimization when Qt is used as the renderer:
             # renderer surface is already a QPixmap
             self.pixmap = self.renderer.surface
-            self.imgwin.set_pixmap(self.pixmap)
 
         else:
             # If we need to build a new pixmap do it here.  We allocate one
@@ -172,45 +228,24 @@ class ImageViewQt(ImageView.ImageViewBase):
                     (self.pixmap.height() < height)):
                 pixmap = QPixmap(width * 2, height * 2)
                 self.pixmap = pixmap
-                self.imgwin.set_pixmap(pixmap)
 
         self.configure(width, height)
 
-    def get_rgb_image_as_buffer(self, output=None, format='png',
-                                quality=90):
-        ibuf = output
-        if ibuf is None:
-            ibuf = BytesIO()
-        imgwin_wd, imgwin_ht = self.get_window_size()
-        qpix = self.pixmap.copy(0, 0,
-                                imgwin_wd, imgwin_ht)
-        qbuf = QtCore.QBuffer()
-        qbuf.open(QtCore.QIODevice.ReadWrite)
-        qpix.save(qbuf, format=format, quality=quality)
-        ibuf.write(bytes(qbuf.data()))
-        qbuf.close()
-        return ibuf
-
     def get_rgb_image_as_widget(self):
-        imgwin_wd, imgwin_ht = self.get_window_size()
-        qpix = self.pixmap.copy(0, 0, imgwin_wd, imgwin_ht)
-        return qpix.toImage()
-
-    def save_rgb_image_as_file(self, filepath, format='png', quality=90):
-        qimg = self.get_rgb_image_as_widget()
-        qimg.save(filepath, format=format, quality=quality)
+        arr = self.renderer.get_surface_as_array(order=self.rgb_order)
+        image = self._get_qimage(arr, self.qimg_fmt)
+        return image
 
     def get_plain_image_as_widget(self):
-        """Used for generating thumbnails.  Does not include overlaid
-        graphics.
+        """Returns a QImage of the drawn images.
+        Does not include overlaid graphics.
         """
         arr = self.getwin_array(order=self.rgb_order)
         image = self._get_qimage(arr, self.qimg_fmt)
         return image
 
     def save_plain_image_as_file(self, filepath, format='png', quality=90):
-        """Used for generating thumbnails.  Does not include overlaid
-        graphics.
+        """Does not include overlaid graphics.
         """
         qimg = self.get_plain_image_as_widget()
         qimg.save(filepath, format=format, quality=quality)
@@ -219,18 +254,36 @@ class ImageViewQt(ImageView.ImageViewBase):
         self._defer_task.stop()
         self._defer_task.start(time_sec)
 
-    def update_image(self):
-        if (not self.pixmap) or (not self.imgwin):
+    def make_context_current(self):
+        ctx = self.imgwin.context()
+        self.imgwin.makeCurrent()
+        return ctx
+
+    def prepare_image(self, cvs_img, cache, whence):
+        self.renderer.prepare_image(cvs_img, cache, whence)
+
+    def update_widget(self):
+        if self.imgwin is None:
             return
 
-        if isinstance(self.renderer.surface, QPixmap):
+        if self.wtype == 'opengl':
+            pass
+
+        elif isinstance(self.renderer.surface, QPixmap):
             # optimization when Qt is used as the renderer:
             # if renderer surface is already an offscreen QPixmap
             # then we can update the window directly from it
-            #self.pixmap = self.renderer.surface
-            pass
+            if self.pixmap is not self.renderer.surface:
+                self.pixmap = self.renderer.surface
+
+            if DEBUG_MODE:
+                qimage = self.pixmap.toImage()
+                save_debug_image(qimage, 'offscreen_image.png', format='png')
 
         else:
+            if self.pixmap is None:
+                return
+
             if isinstance(self.renderer.surface, QImage):
                 # optimization when Qt is used as the renderer:
                 # renderer surface is already a QImage
@@ -249,9 +302,7 @@ class ImageViewQt(ImageView.ImageViewBase):
 
             # copy image from renderer to offscreen pixmap
             painter = get_painter(self.pixmap)
-            #painter.setWorldMatrixEnabled(True)
 
-            # fill surface with background color
             size = self.pixmap.size()
             width, height = size.width(), size.height()
 
@@ -260,6 +311,9 @@ class ImageViewQt(ImageView.ImageViewBase):
                               qimage,
                               QtCore.QRect(0, 0, width, height))
 
+            if DEBUG_MODE:
+                save_debug_image(qimage, 'offscreen_image.png', format='png')
+
         self.logger.debug("updating window from pixmap")
         if hasattr(self, 'scene'):
             imgwin_wd, imgwin_ht = self.get_window_size()
@@ -396,6 +450,10 @@ class RenderGraphicsViewZoom(RenderMixin, RenderGraphicsView):
     pass
 
 
+class RenderGLWidgetZoom(RenderMixin, RenderGLWidget):
+    pass
+
+
 class QtEventMixin(object):
 
     def __init__(self):
@@ -530,14 +588,14 @@ class QtEventMixin(object):
         keyname2 = "%s" % (event.text())
         keyname = self.transkey(keyname, keyname2)
         self.logger.debug("key press event, key=%s" % (keyname))
-        return self.make_ui_callback('key-press', keyname)
+        return self.make_ui_callback_viewer(self, 'key-press', keyname)
 
     def key_release_event(self, widget, event):
         keyname = event.key()
         keyname2 = "%s" % (event.text())
         keyname = self.transkey(keyname, keyname2)
         self.logger.debug("key release event, key=%s" % (keyname))
-        return self.make_ui_callback('key-release', keyname)
+        return self.make_ui_callback_viewer(self, 'key-release', keyname)
 
     def button_press_event(self, widget, event):
         buttons = event.buttons()
@@ -559,7 +617,8 @@ class QtEventMixin(object):
 
         data_x, data_y = self.check_cursor_location()
 
-        return self.make_ui_callback('button-press', button, data_x, data_y)
+        return self.make_ui_callback_viewer(self, 'button-press', button,
+                                            data_x, data_y)
 
     def button_release_event(self, widget, event):
         # note: for mouseRelease this needs to be button(), not buttons()!
@@ -581,7 +640,8 @@ class QtEventMixin(object):
 
         data_x, data_y = self.check_cursor_location()
 
-        return self.make_ui_callback('button-release', button, data_x, data_y)
+        return self.make_ui_callback_viewer(self, 'button-release', button,
+                                            data_x, data_y)
 
     def motion_notify_event(self, widget, event):
 
@@ -608,7 +668,8 @@ class QtEventMixin(object):
 
         data_x, data_y = self.check_cursor_location()
 
-        return self.make_ui_callback('motion', button, data_x, data_y)
+        return self.make_ui_callback_viewer(self, 'motion', button,
+                                            data_x, data_y)
 
     def scroll_event(self, widget, event):
         x, y = event.x(), event.y()
@@ -632,16 +693,16 @@ class QtEventMixin(object):
                 dx, dy = point.x(), point.y()
 
                 # Synthesize this as a pan gesture event
-                self.make_ui_callback('pan', 'start', 0, 0)
-                self.make_ui_callback('pan', 'move', dx, dy)
-                return self.make_ui_callback('pan', 'stop', 0, 0)
+                self.make_ui_callback_viewer(self, 'pan', 'start', 0, 0)
+                self.make_ui_callback_viewer(self, 'pan', 'move', dx, dy)
+                return self.make_ui_callback_viewer(self, 'pan', 'stop', 0, 0)
 
         num_degrees, direction = get_scroll_info(event)
         self.logger.debug("scroll deg={} direction={}".format(
             num_degrees, direction))
 
-        return self.make_ui_callback('scroll', direction, num_degrees,
-                                     data_x, data_y)
+        return self.make_ui_callback_viewer(self, 'scroll', direction,
+                                            num_degrees, data_x, data_y)
 
     def gesture_event(self, widget, event):
         gesture = event.gestures()[0]
@@ -697,7 +758,8 @@ class QtEventMixin(object):
             self.logger.debug("swipe gesture hdir=%s vdir=%s" % (
                 hdir, vdir))
 
-            return self.make_ui_callback('swipe', gstate, hdir, vdir)
+            return self.make_ui_callback_viewer(self, 'swipe', gstate,
+                                                hdir, vdir)
 
     def gs_pinching(self, event, gesture, gstate):
         rot = gesture.rotationAngle()
@@ -705,7 +767,7 @@ class QtEventMixin(object):
         self.logger.debug("pinch gesture rot=%f scale=%f state=%s" % (
             rot, scale, gstate))
 
-        return self.make_ui_callback('pinch', gstate, rot, scale)
+        return self.make_ui_callback_viewer(self, 'pinch', gstate, rot, scale)
 
     def gs_panning(self, event, gesture, gstate):
         d = gesture.delta()
@@ -713,7 +775,7 @@ class QtEventMixin(object):
         self.logger.debug("pan gesture dx=%f dy=%f state=%s" % (
             dx, dy, gstate))
 
-        return self.make_ui_callback('pan', gstate, dx, dy)
+        return self.make_ui_callback_viewer(self, 'pan', gstate, dx, dy)
 
     def gs_tapping(self, event, gesture, gstate):
         self.logger.debug("tapping gesture state=%s" % (
@@ -748,7 +810,7 @@ class QtEventMixin(object):
 
         event.setAccepted(True)
         #event.acceptProposedAction()
-        self.make_ui_callback('drag-drop', data)
+        self.make_ui_callback_viewer(self, 'drag-drop', data)
 
 
 class ImageViewEvent(QtEventMixin, ImageViewQt):
@@ -758,7 +820,9 @@ class ImageViewEvent(QtEventMixin, ImageViewQt):
                              settings=settings, render=render)
 
         # replace the widget our parent provided
-        if self.wtype == 'scene':
+        if self.wtype == 'opengl':
+            imgwin = RenderGLWidgetZoom()
+        elif self.wtype == 'scene':
             imgwin = RenderGraphicsViewZoom()
             imgwin.setScene(self.scene)
         else:
@@ -791,7 +855,7 @@ class ImageViewZoom(Mixins.UIMixin, ImageViewEvent):
                                 rgbmap=rgbmap, render=render)
         Mixins.UIMixin.__init__(self)
 
-        self.ui_set_active(True)
+        self.ui_set_active(True, viewer=self)
 
         if bindmap is None:
             bindmap = ImageViewZoom.bindmapClass(self.logger)
@@ -980,4 +1044,6 @@ class ScrolledView(QtGui.QAbstractScrollArea):
             raise ValueError("Bad scroll bar option: '%s'; should be one of ('on', 'off' or 'auto')" % (vertical))
 
 
-#END
+def save_debug_image(qimage, filename, format='png'):
+    path = os.path.join(tempfile.gettempdir(), filename)
+    qimage.save(path, format=format)
