@@ -8,6 +8,7 @@ import glob
 import os
 import math
 import weakref
+import time
 
 import ginga.toolkit
 from ginga.util import iohelper
@@ -40,7 +41,11 @@ qtpy_import_error = ""
 try:
     from qtpy import QtCore
     from qtpy import QtWidgets as QtGui
-    from qtpy.QtGui import QImage, QColor, QFont, QPixmap, QIcon, QCursor, QPainter, QPen, QPolygonF, QPolygon, QTextCursor, QDrag, QPainterPath, QBrush, QFontDatabase  # noqa
+    from qtpy.QtGui import (QImage, QColor, QFont, QPixmap, QIcon,  # noqa
+                            QPainter, QPen, QPolygonF, QPolygon, QTextCursor,
+                            QDrag, QPainterPath, QBrush, QFontDatabase,
+                            QCursor, QFontMetrics, QSurfaceFormat)
+    from qtpy.QtWidgets import QOpenGLWidget  # noqa
     from qtpy.QtCore import QItemSelectionModel  # noqa
     from qtpy.QtWidgets import QApplication  # noqa
     try:
@@ -265,6 +270,8 @@ class Timer(Callback.Callbacks):
         self.timer.setSingleShot(True)
         self.timer.setTimerType(QtCore.Qt.PreciseTimer)
         self.timer.timeout.connect(self._expired_cb)
+        self.start_time = 0.0
+        self.deadline = 0.0
 
         for name in ('expired', 'canceled'):
             self.enable_callback(name)
@@ -281,6 +288,8 @@ class Timer(Callback.Callbacks):
     def set(self, duration):
         self.stop()
 
+        self.start_time = time.time()
+        self.deadline = self.start_time + duration
         # QTimer set in milliseconds
         ms = int(duration * 1000.0)
         self.timer.start(ms)
@@ -288,6 +297,27 @@ class Timer(Callback.Callbacks):
     def _expired_cb(self):
         self.make_callback('expired')
 
+    def is_set(self):
+        return self.timer.isActive()
+
+    def cond_set(self, time_sec):
+        if not self.is_set():
+            # TODO: probably a race condition here
+            self.set(time_sec)
+
+    def elapsed_time(self):
+        return time.time() - self.start_time
+
+    def time_left(self):
+        #return max(0.0, self.deadline - time.time())
+        # remainingTime() returns value in msec, or -1 if timer is not set
+        t = self.timer.remainingTime()
+        t = max(0.0, t)
+        return t / 1000.0
+
+    def get_deadline(self):
+        return self.deadline
+
     def stop(self):
         try:
             self.timer.stop()
@@ -402,14 +432,24 @@ _painters = weakref.WeakKeyDictionary()
 
 
 def get_painter(surface):
-    if surface in _painters:
-        return _painters[surface]
+    # QImage is not hashable
+    if not isinstance(surface, QImage):
+        if surface in _painters:
+            return _painters[surface]
 
     painter = QPainter(surface)
     painter.setRenderHint(QPainter.Antialiasing)
     painter.setRenderHint(QPainter.TextAntialiasing)
-    _painters[surface] = painter
+    if not isinstance(surface, QImage):
+        _painters[surface] = painter
     return painter
 
 
+def set_default_opengl_context():
+    from ginga.opengl.glsl import req
+    fmt = QSurfaceFormat()
+    fmt.setVersion(req.major, req.minor)
+    fmt.setProfile(QSurfaceFormat.CoreProfile)
+    fmt.setDefaultFormat(fmt)
+
 # END
