@@ -1,26 +1,28 @@
 from ginga.qtw.ImageViewQt import CanvasView, ScrolledView
-from ginga.qtw.ImageViewCanvasQt import ImageViewCanvas
+from ginga.qtw.ImageViewCanvasQt import ImageViewCanvas  # noqa
 from ginga.qtw import Widgets
 
+
 class GingaViewerWidget(Widgets.WidgetBase):
 
     def __init__(self, viewer=None, width=600, height=600):
         super(GingaViewerWidget, self).__init__()
 
         if viewer is None:
-            viewer = CanvasView(logger)
+            viewer = CanvasView()
         self.logger = viewer.logger
 
         self.viewer = viewer
         self.widget = viewer.get_widget()
 
+
 class GingaScrolledViewerWidget(Widgets.WidgetBase):
 
     def __init__(self, viewer=None, width=600, height=600):
         super(GingaScrolledViewerWidget, self).__init__()
 
         if viewer is None:
-            viewer = CanvasView(logger)
+            viewer = CanvasView()
         self.logger = viewer.logger
 
         self.viewer = viewer
@@ -30,4 +32,4 @@ class GingaScrolledViewerWidget(Widgets.WidgetBase):
         self.widget.scroll_bars(horizontal=horizontal,
                                 vertical=vertical)
 
-#END
+# END
