@@ -20,7 +20,7 @@ from ginga.misc.Bunch import Bunch
 from ginga.misc import Task, ModuleManager, Settings, log
 import ginga.version as version
 import ginga.toolkit as ginga_toolkit
-from ginga.util import paths, rgb_cms
+from ginga.util import paths, rgb_cms, json
 
 # Catch warnings
 logging.captureWarnings(True)
@@ -32,7 +32,7 @@ default_layout = ['seq', {},
                     dict(row=['hbox', dict(name='menu')],
                          stretch=0),
                     dict(row=['hpanel', dict(name='hpnl'),
-                     ['ws', dict(name='left', wstype='tabs',
+                     ['ws', dict(name='left', wstype='tabs',  # noqa
                                  width=300, height=-1, group=2),
                       # (tabname, layout), ...
                       [("Info", ['vpanel', {},
@@ -44,7 +44,8 @@ default_layout = ['seq', {},
                         )]],
                      ['vbox', dict(name='main', width=600),
                       dict(row=['ws', dict(name='channels', wstype='tabs',
-                                           group=1, use_toolbar=True)],
+                                           group=1, use_toolbar=True,
+                                           default=True)],
                            stretch=1),
                       dict(row=['ws', dict(name='cbar', wstype='stack',
                                            group=99)], stretch=0),
@@ -61,7 +62,7 @@ default_layout = ['seq', {},
                                     ]
                         )]
                       ],
-                     ], stretch=1),
+                     ], stretch=1),  # noqa
                     dict(row=['ws', dict(name='toolbar', wstype='stack',
                                          height=40, group=2)],
                          stretch=0),
@@ -144,6 +145,7 @@ plugins = [
     Bunch(module='ChangeHistory', tab='History', workspace='right',
           menu="History [G]", start=False, category='Utils', ptype='global'),
     Bunch(module='Mosaic', workspace='dialogs', category='Utils', ptype='local'),
+    Bunch(module='Collage', workspace='dialogs', category='Utils', ptype='local'),
     Bunch(module='FBrowser', tab='Open File', workspace='right',
           menu="Open File [G]", start=False, category='Utils', ptype='global'),
     Bunch(module='Preferences', workspace='dialogs', category='Utils',
@@ -168,10 +170,11 @@ class ReferenceViewer(object):
     This class exists solely to be able to customize the reference
     viewer startup.
     """
-    def __init__(self, layout=default_layout):
-        self.plugins = []
+    def __init__(self, layout=default_layout, plugins=plugins):
         self.layout = layout
         self.channels = ['Image']
+        self.default_plugins = plugins
+        self.plugins = []
 
     def add_plugin_spec(self, spec):
         self.plugins.append(spec)
@@ -184,8 +187,8 @@ class ReferenceViewer(object):
         Add the ginga-distributed default set of plugins to the
         reference viewer.
         """
-        # add default global plugins
-        for spec in plugins:
+        # add default plugins
+        for spec in self.default_plugins:
             ptype = spec.get('ptype', 'local')
             if ptype == 'global' and spec.module not in except_global:
                 self.add_plugin_spec(spec)
@@ -229,6 +232,8 @@ class ReferenceViewer(object):
             # newer ArgParse
             add_argument = argprs.add_argument
 
+        add_argument("--basedir", dest="basedir", metavar="NAME",
+                     help="Specify Ginga configuration area")
         add_argument("--bufsize", dest="bufsize", metavar="NUM",
                      type=int, default=10,
                      help="Buffer length to NUM")
@@ -265,6 +270,9 @@ class ReferenceViewer(object):
         add_argument("--opencl", dest="opencl", default=False,
                      action="store_true",
                      help="Use OpenCL acceleration")
+        add_argument("--opengl", dest="opengl", default=False,
+                     action="store_true",
+                     help="Use OpenGL acceleration")
         add_argument("--plugins", dest="plugins", metavar="NAMES",
                      help="Specify additional plugins to load")
         add_argument("--profile", dest="profile", action="store_true",
@@ -296,6 +304,9 @@ class ReferenceViewer(object):
         # Create a logger
         logger = log.get_logger(name='ginga', options=options)
 
+        if options.basedir is not None:
+            paths.ginga_home = os.path.expanduser(options.basedir)
+
         # Get settings (preferences)
         basedir = paths.ginga_home
         if not os.path.exists(basedir):
@@ -317,6 +328,9 @@ class ReferenceViewer(object):
                               icc_working_profile=None,
                               font_scaling_factor=None,
                               save_layout=True,
+                              use_opengl=False,
+                              layout_file='layout',
+                              plugin_file='plugins.json',
                               channel_prefix="Image")
         settings.load(onError='silent')
 
@@ -443,6 +457,9 @@ class ReferenceViewer(object):
                 logger.warning(
                     "failed to set OpenCL preference: %s" % (str(e)))
 
+        if options.opengl:
+            settings.set(use_opengl=True)
+
         # Create the dynamic module manager
         mm = ModuleManager.ModuleManager(logger)
 
@@ -456,20 +473,10 @@ class ReferenceViewer(object):
         ginga_shell = GingaShell(logger, thread_pool, mm, prefs,
                                  ev_quit=ev_quit)
 
-        # user wants to set font scaling.
-        # NOTE: this happens *after* creation of shell object, since
-        # Application object constructor will also set this
-        font_scaling = settings.get('font_scaling_factor', None)
-        if font_scaling is not None:
-            logger.debug("overriding font_scaling_factor to {}".format(font_scaling))
-            from ginga.fonts import font_asst
-            font_asst.default_scaling_factor = font_scaling
-
-        layout_file = None
-        if not options.norestore and settings.get('save_layout', False):
-            layout_file = os.path.join(basedir, 'layout')
-
-        ginga_shell.set_layout(self.layout, layout_file=layout_file)
+        layout_file = os.path.join(basedir, settings.get('layout_file',
+                                                         'layout'))
+        ginga_shell.set_layout(self.layout, layout_file=layout_file,
+                               save_layout=settings.get('save_layout', True))
 
         # User configuration (custom star catalogs, etc.)
         if have_ginga_config:
@@ -489,7 +496,21 @@ class ReferenceViewer(object):
                 logger.error("Traceback:\n%s" % (tb_str))
 
         # Build desired layout
-        ginga_shell.build_toplevel()
+        ginga_shell.build_toplevel(ignore_saved_layout=options.norestore)
+
+        # Does user have a customized plugin setup?  If so, override the
+        # default plugins to be that
+        plugin_file = settings.get('plugin_file', None)
+        if plugin_file is not None:
+            plugin_file = os.path.join(basedir, plugin_file)
+            if os.path.exists(plugin_file):
+                logger.info("Reading plugin file '%s'..." % (plugin_file))
+                try:
+                    with open(plugin_file, 'r') as in_f:
+                        buf = in_f.read()
+                        self.plugins = json.loads(buf)
+                except Exception as e:
+                    logger.error("Error reading plugin file: %s" % (str(e)))
 
         # Did user specify a particular geometry?
         if options.geometry:
@@ -554,10 +575,13 @@ class ReferenceViewer(object):
                          hidden=False, pfx=pfx)
             self.add_plugin_spec(spec)
 
-        # Add non-disabled plugins
-        enabled_plugins = [spec for spec in self.plugins
-                           if spec.module.lower() not in disabled_plugins]
-        ginga_shell.set_plugins(enabled_plugins)
+        # Mark disabled plugins
+        for spec in self.plugins:
+            if spec.get('enabled', None) is None:
+                spec['enabled'] = (False if spec.module.lower() in disabled_plugins
+                                   else True)
+        # submit plugin specs to shell
+        ginga_shell.set_plugins(self.plugins)
 
         # start any plugins that have start=True
         ginga_shell.boot_plugins()
