@@ -341,7 +341,7 @@ class Catalogs(GingaPlugin.LocalPlugin):
         # turn off any mode user may be in
         self.modes_off()
 
-        self.canvas.ui_set_active(True)
+        self.canvas.ui_set_active(True, viewer=self.fitsimage)
         #self.fv.show_status("Draw a rectangle with the right mouse button")
 
     def stop(self):
