@@ -1,60 +1,53 @@
-#
-# ChangeHistory.py -- ChangeHistory global plugin for Ginga.
-#
-from ginga import GingaPlugin
-from ginga.gw import Widgets
-from ginga.misc import Bunch
-from ginga.util.iohelper import shorten_name
+# This is open-source software licensed under a BSD license.
+# Please see the file LICENSE.txt for details.
+"""
+Keep track of buffer change history.
 
+**Plugin Type: Global**
 
-class ChangeHistory(GingaPlugin.GlobalPlugin):
-    """
-    ChangeHistory
-    =============
-    Keep track of buffer change history.
+``ChangeHistory`` is a global plugin.  Only one instance can be opened.
 
-    Plugin Type: Global
-    -------------------
-    ChangeHistory is a global plugin.  Only one instance can be opened.
+This plugin is used to log any changes to data buffer. For example,
+a change log would appear here if a new image is added to a mosaic via the
+``Mosaic`` plugin. Like ``Contents``,
+the log is sorted by channel, and then by image name.
 
-    This plugin is used to log any changes to data buffer. For example,
-    a change log would appear here if a new image is added to a mosaic via the
-    `Mosaic` plugin. Like `Contents`, the log is sorted by channel, and then
-    by image name.
+**Usage**
 
-    Notes
-    -----
-    History should stay no matter what channel or image is active.
-    New history can be added, but old history cannot be deleted,
-    unless the image/channel itself is deleted.
+History should stay no matter what channel or image is active.
+New history can be added, but old history cannot be deleted,
+unless the image/channel itself is deleted.
 
-    The `redo()` method picks up a ``'modified'`` event and displays
-    related metadata here. The metadata is obtained as follows:
-
-    .. code-block:: python
+The ``redo()`` method picks up an ``'add-image-info'`` event and displays
+related metadata here. The metadata is obtained as follows::
 
         channel = self.fv.get_channel_info(chname)
         iminfo = channel.get_image_info(imname)
         timestamp = iminfo.time_modified
         description = iminfo.reason_modified  # Optional
 
-    While ``'time_modified'`` is automatically added by Ginga,
-    ``'reason_modified'`` is optional and has be to explicitly set
-    by the calling plugin in the same method that issues the
-    ``'modified'`` callback, like this:
+Both ``'time_modified'`` and ``'reason_modified'`` have to be explicitly set
+by the calling plugin in the same method that issues the
+``'add-image-info'`` callback, like this::
 
-    .. code-block:: python
-
-        # This issues the 'modified' callback and sets the timestamp
+        # This changes the data buffer
         image.set_data(new_data, ...)
+        # Add description for ChangeHistory
+        info = dict(time_modified=datetime.utcnow(),
+                    reason_modified='Data has changed')
+        self.fv.update_image_info(image, info)
 
-        # Manually add the description
-        chname = self.fv.get_channel_name(self.fitsimage)
-        channel = self.fv.get_channel_info(chname)
-        iminfo = channel.get_image_info(image.get('name'))
-        iminfo.reason_modified = 'Something was done to this image buffer'
+"""
+from ginga import GingaPlugin
+from ginga.gw import Widgets
+from ginga.misc import Bunch
+from ginga.util.iohelper import shorten_name
+
+__all__ = ['ChangeHistory']
+
+
+class ChangeHistory(GingaPlugin.GlobalPlugin):
 
-    """
     def __init__(self, fv):
         # superclass defines some variables for us, like logger
         super(ChangeHistory, self).__init__(fv)
@@ -197,6 +190,16 @@ class ChangeHistory(GingaPlugin.GlobalPlugin):
         timestamp = iminfo.time_modified
 
         if timestamp is None:
+            reason = iminfo.get('reason_modified', None)
+            if reason is not None:
+                self.fv.show_error(
+                    "{0} invoked 'modified' callback to ChangeHistory with a "
+                    "reason but without a timestamp. The plugin invoking the "
+                    "callback is no longer be compatible with Ginga. "
+                    "Please contact plugin developer to update the plugin "
+                    "to use self.fv.update_image_info() like Mosaic "
+                    "plugin.".format(imname))
+
             # Image somehow lost its history
             self.remove_image_info_cb(self.fv, channel, iminfo)
             return
@@ -300,8 +303,7 @@ class ChangeHistory(GingaPlugin.GlobalPlugin):
         return 'changehistory'
 
 
-# Replace module docstring with config doc for auto insert by Sphinx.
-# In the future, if we need the real docstring, we can append instead of
-# overwrite.
+# Append module docstring with config doc for auto insert by Sphinx.
 from ginga.util.toolbox import generate_cfg_example  # noqa
-__doc__ = generate_cfg_example('plugin_ChangeHistory', package='ginga')
+if __doc__ is not None:
+    __doc__ += generate_cfg_example('plugin_ChangeHistory', package='ginga')
