@@ -119,7 +119,7 @@ class PixTable(GingaPlugin.LocalPlugin):
 
         self.dc = self.fv.get_draw_classes()
         canvas = self.dc.DrawingCanvas()
-        canvas.set_callback('cursor-changed', self.cursor_cb)
+        self.fitsimage.set_callback('cursor-changed', self.cursor_cb)
         canvas.enable_draw(True)
         canvas.set_drawtype('point', color='cyan', linestyle='dash')
         canvas.set_callback('draw-event', self.draw_cb)
@@ -402,6 +402,8 @@ class PixTable(GingaPlugin.LocalPlugin):
         width, height = self.fitsimage.get_dims(data)
         if self.txt_arr is None:
             return
+        if data.shape != self.txt_arr.shape:
+            return
 
         maxval = np.nanmax(data)
         minval = np.nanmin(data)
@@ -463,25 +465,28 @@ class PixTable(GingaPlugin.LocalPlugin):
         # turn off any mode user may be in
         self.modes_off()
 
-        self.canvas.ui_set_active(True)
+        self.canvas.ui_set_active(True, viewer=self.fitsimage)
         self.redo()
 
     def redo(self):
         if self.pixview is None:
             return
         # cut out and set the pixel table data
-        image = self.fitsimage.get_image()
+        image = self.fitsimage.get_vip()
 
         if image is None:
             return
 
         # We report the value across the pixel, even though the coords
         # change halfway across the pixel
-        data_x, data_y = int(self.lastx + 0.5), int(self.lasty + 0.5)
+        px_off = self.fitsimage.data_off
+        data_x, data_y = (int(np.floor(self.lastx + px_off)),
+                          int(np.floor(self.lasty + px_off)))
 
         # cutout image data
         data, x1, y1, x2, y2 = image.cutout_radius(data_x, data_y,
                                                    self.pixtbl_radius)
+
         self.fv.error_wrap(self.plot, data, x1, y1, x2, y2,
                            self.lastx, self.lasty,
                            self.pixtbl_radius, maxv=9)
@@ -546,6 +551,8 @@ class PixTable(GingaPlugin.LocalPlugin):
         self._rebuild_table()
 
     def cursor_cb(self, canvas, junk, data_x, data_y):
+        if not self.gui_up:
+            return
         if self.mark_selected is not None:
             return False
         if self.pixview is None:
