@@ -731,7 +731,7 @@ class TVMark(LocalPlugin):
         # turn off any mode user may be in
         self.modes_off()
 
-        self.canvas.ui_set_active(True)
+        self.canvas.ui_set_active(True, viewer=self.fitsimage)
         self.fv.show_status('Press "Help" for instructions')
 
     def stop(self):
