@@ -4,9 +4,6 @@
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
-from ginga.util.six import itervalues
-from ginga.util.six.moves import zip
-
 import logging
 from collections import OrderedDict
 
@@ -32,6 +29,13 @@ class TableViewBase(Callback.Callbacks):
     vname = 'Ginga Table'
     vtypes = [AstroTable.AstroTable]
 
+    @classmethod
+    def viewable(cls, dataobj):
+        """Test whether `dataobj` is viewable by this viewer."""
+        if not isinstance(dataobj, AstroTable.AstroTable):
+            return False
+        return True
+
     def __init__(self, logger=None, settings=None):
         Callback.Callbacks.__init__(self)
 
@@ -48,8 +52,8 @@ class TableViewBase(Callback.Callbacks):
         # for debugging
         self.name = str(self)
 
-        self.settings.addDefaults(color_alternate_rows=True,
-                                  max_rows_for_col_resize=5000)
+        self.settings.add_defaults(color_alternate_rows=True,
+                                   max_rows_for_col_resize=5000)
 
         # For callbacks
         for name in ('table-set', 'configure', ):
@@ -89,10 +93,14 @@ class TableViewBase(Callback.Callbacks):
     def get_table(self):
         return self._table
 
-    # for compatibility with ImageView
+    # TO BE DEPRECATED
     get_image = get_table
     set_image = set_table
 
+    # for compatibility with other Ginga viewers
+    get_dataobj = get_table
+    set_dataobj = set_table
+
     def initialize_channel(self, fv, channel):
         """The reference viewer calls this method with itself and the channel
         when it is inserted into a channel.
@@ -101,6 +109,8 @@ class TableViewBase(Callback.Callbacks):
 
 
 class TableViewGw(TableViewBase):
+    """A Ginga viewer for displaying tables of FITS data.
+    """
 
     def __init__(self, logger=None, settings=None):
         super(TableViewGw, self).__init__(logger=logger, settings=settings)
@@ -137,7 +147,7 @@ class TableViewGw(TableViewBase):
 
         # Table header with units
         columns = [('Row', '_DISPLAY_ROW')]
-        for c in itervalues(a_tab.columns):
+        for c in a_tab.columns.values():
             col_str = '{0:^s}\n{1:^s}'.format(c.name, str(c.unit))
             columns.append((col_str, c.name))
 
