@@ -106,7 +106,7 @@ class ImageViewBokeh(ImageViewSS):
     def get_figure(self):
         return self.figure
 
-    def update_image(self):
+    def update_widget(self):
         if self.figure is None:
             return
 
@@ -266,7 +266,7 @@ class ImageViewBokeh2(ImageView.ImageViewBase):
         self.figure.write_to_png(ibuf)
         return ibuf
 
-    def update_image(self):
+    def update_widget(self):
         self.logger.debug("redraw surface")
         if self.figure is None:
             return
@@ -478,14 +478,14 @@ class ImageViewEvent(ImageViewBokeh):
         keyname = self.transkey(keyname)
         if keyname is not None:
             self.logger.debug("key press event, key=%s" % (keyname))
-            return self.make_ui_callback('key-press', keyname)
+            return self.make_ui_callback_viewer(self, 'key-press', keyname)
 
     def key_release_event(self, event):
         keyname = event.key
         keyname = self.transkey(keyname)
         if keyname is not None:
             self.logger.debug("key release event, key=%s" % (keyname))
-            return self.make_ui_callback('key-release', keyname)
+            return self.make_ui_callback_viewer(self, 'key-release', keyname)
 
     def button_press_event(self, event):
         x, y = int(event.x), int(event.y)
@@ -494,7 +494,9 @@ class ImageViewEvent(ImageViewBokeh):
         ##     button |= 0x1 << (event.button - 1)
         self.logger.debug("button event at %dx%d, button=%x" % (x, y, button))
 
-        data_x, data_y = self.get_data_xy(x, y)
+        self.last_win_x, self.last_win_y = x, y
+        data_x, data_y = self.check_cursor_location()
+
         return self.make_ui_callback('button-press', button, data_x, data_y)
 
     def button_release_event(self, event):
@@ -504,7 +506,9 @@ class ImageViewEvent(ImageViewBokeh):
             button |= 0x1 << (event.button - 1)
         self.logger.debug("button release at %dx%d button=%x" % (x, y, button))
 
-        data_x, data_y = self.get_data_xy(x, y)
+        self.last_win_x, self.last_win_y = x, y
+        data_x, data_y = self.check_cursor_location()
+
         return self.make_ui_callback('button-release', button, data_x, data_y)
 
     def select_event_cb(self, attrname, old_val, new_val):
@@ -526,11 +530,14 @@ class ImageViewEvent(ImageViewBokeh):
         ##     button |= 0x1 << (event.button - 1)
         self.logger.debug("motion event at %dx%d, button=%x" % (x, y, button))
 
-        data_x, data_y = self.get_data_xy(x, y)
+        self.last_win_x, self.last_win_y = x, y
+        data_x, data_y = self.check_cursor_location()
+
         self.last_data_x, self.last_data_y = data_x, data_y
         self.logger.info("motion event at DATA %dx%d" % (data_x, data_y))
 
-        return self.make_ui_callback('motion', button, data_x, data_y)
+        return self.make_ui_callback_viewer(self, 'motion', button,
+                                            data_x, data_y)
 
     def scroll_event(self, event):
         x, y = int(event.x), int(event.y)
@@ -549,11 +556,13 @@ class ImageViewEvent(ImageViewBokeh):
         self.logger.info("scroll deg=%f direction=%f" % (
             amount, direction))
 
-        data_x, data_y = self.get_data_xy(x, y)
+        self.last_win_x, self.last_win_y = x, y
+        data_x, data_y = self.check_cursor_location()
+
         self.last_data_x, self.last_data_y = data_x, data_y
 
-        return self.make_ui_callback('scroll', direction, amount,
-                                     data_x, data_y)
+        return self.make_ui_callback_viewer(self, 'scroll', direction, amount,
+                                            data_x, data_y)
 
     def pinch_event(self, event):
         # no rotation (seemingly) in the Bokeh pinch event
@@ -561,19 +570,19 @@ class ImageViewEvent(ImageViewBokeh):
         scale = event.scale
         self.logger.debug("pinch gesture rot=%f scale=%f" % (rot, scale))
 
-        return self.make_ui_callback('pinch', 'move', rot, scale)
+        return self.make_ui_callback_viewer(self, 'pinch', 'move', rot, scale)
 
     def pan_start_event(self, event):
         dx, dy = int(event.delta_x), int(event.delta_y)
         self.logger.debug("pan gesture dx=%f dy=%f" % (dx, dy))
 
-        return self.make_ui_callback('pan', 'start', dx, dy)
+        return self.make_ui_callback_viewer(self, 'pan', 'start', dx, dy)
 
     def pan_event(self, event):
         dx, dy = int(event.delta_x), int(event.delta_y)
         self.logger.debug("pan gesture dx=%f dy=%f" % (dx, dy))
 
-        return self.make_ui_callback('pan', 'move', dx, dy)
+        return self.make_ui_callback_viewer(self, 'pan', 'move', dx, dy)
 
     def tap_event(self, event):
         x, y = int(event.x), int(event.y)
@@ -582,7 +591,9 @@ class ImageViewEvent(ImageViewBokeh):
         ##     button |= 0x1 << (event.button - 1)
         self.logger.debug("tap event at %dx%d, button=%x" % (x, y, button))
 
-        data_x, data_y = self.get_data_xy(x, y)
+        self.last_win_x, self.last_win_y = x, y
+        data_x, data_y = self.check_cursor_location()
+
         return self.make_ui_callback('button-press', button, data_x, data_y)
 
     def press_event(self, event):
@@ -610,7 +621,7 @@ class ImageViewZoom(Mixins.UIMixin, ImageViewEvent):
                                 settings=settings)
         Mixins.UIMixin.__init__(self)
 
-        self.ui_set_active(True)
+        self.ui_set_active(True, viewer=self)
 
         if bindmap is None:
             bindmap = ImageViewZoom.bindmapClass(self.logger)
