@@ -18,7 +18,6 @@ See example usage with an ipython notebook at:
 You will need a reasonably modern web browser with HTML5 canvas support.
 Tested with Chromium 41.0.2272.76, Firefox 37.0.2, Safari 7.1.6
 """
-from __future__ import print_function
 
 import sys
 import os
@@ -34,6 +33,7 @@ from ginga.misc import log, Task, Bunch
 from ginga.Bindings import ImageViewBindings
 from ginga.misc.Settings import SettingGroup
 from ginga.util.paths import ginga_home
+from ginga.util import loader
 
 from ginga.web.pgw import js, PgHelp, Widgets, Viewers
 
@@ -98,9 +98,7 @@ class BasicCanvasView(Viewers.CanvasView):
         """
         Load a FITS file into the viewer.
         """
-        image = AstroImage.AstroImage(logger=self.logger)
-        image.load_file(filepath)
-
+        image = loader.load_data(filepath, logger=self.logger)
         self.set_image(image)
 
     load = load_fits
@@ -513,42 +511,41 @@ def main(options, args):
 
 if __name__ == "__main__":
 
-    # Parse command line options with nifty optparse module
-    from optparse import OptionParser
+    # Parse command line options
+    from argparse import ArgumentParser
 
-    usage = "usage: %prog [options] cmd [args]"
-    optprs = OptionParser(usage=usage, version=('%%prog'))
+    argprs = ArgumentParser()
 
-    optprs.add_option("-d", "--basedir", dest="basedir", metavar="DIR",
-                      default=".",
-                      help="Directory which is at the base of file open requests")
-    optprs.add_option("--debug", dest="debug", default=False, action="store_true",
-                      help="Enter the pdb debugger on main()")
-    optprs.add_option("--host", dest="host", metavar="HOST",
-                      default="localhost",
-                      help="HOST used to decide which interfaces to listen on")
-    optprs.add_option("--log", dest="logfile", metavar="FILE",
-                      help="Write logging output to FILE")
-    optprs.add_option("--loglevel", dest="loglevel", metavar="LEVEL",
-                      type='int', default=logging.INFO,
-                      help="Set logging level to LEVEL")
-    optprs.add_option("--numthreads", dest="numthreads", type="int",
-                      default=5, metavar="NUM",
-                      help="Start NUM threads in thread pool")
-    optprs.add_option("--stderr", dest="logstderr", default=False,
-                      action="store_true",
-                      help="Copy logging also to stderr")
-    optprs.add_option("--opencv", dest="use_opencv", default=False,
-                      action="store_true",
-                      help="Use OpenCv acceleration")
-    optprs.add_option("-p", "--port", dest="port",
-                      type='int', default=9909, metavar="PORT",
-                      help="Default PORT to use for the web socket")
-    optprs.add_option("--profile", dest="profile", action="store_true",
-                      default=False,
-                      help="Run the profiler on main()")
+    argprs.add_argument("-d", "--basedir", dest="basedir", metavar="DIR",
+                        default=".",
+                        help="Directory which is at the base of file open requests")
+    argprs.add_argument("--debug", dest="debug", default=False, action="store_true",
+                        help="Enter the pdb debugger on main()")
+    argprs.add_argument("--host", dest="host", metavar="HOST",
+                        default="localhost",
+                        help="HOST used to decide which interfaces to listen on")
+    argprs.add_argument("--log", dest="logfile", metavar="FILE",
+                        help="Write logging output to FILE")
+    argprs.add_argument("--loglevel", dest="loglevel", metavar="LEVEL",
+                        type=int, default=logging.INFO,
+                        help="Set logging level to LEVEL")
+    argprs.add_argument("--numthreads", dest="numthreads", type=int,
+                        default=5, metavar="NUM",
+                        help="Start NUM threads in thread pool")
+    argprs.add_argument("--stderr", dest="logstderr", default=False,
+                        action="store_true",
+                        help="Copy logging also to stderr")
+    argprs.add_argument("--opencv", dest="use_opencv", default=False,
+                        action="store_true",
+                        help="Use OpenCv acceleration")
+    argprs.add_argument("-p", "--port", dest="port",
+                        type=int, default=9909, metavar="PORT",
+                        help="Default PORT to use for the web socket")
+    argprs.add_argument("--profile", dest="profile", action="store_true",
+                        default=False,
+                        help="Run the profiler on main()")
 
-    (options, args) = optprs.parse_args(sys.argv[1:])
+    (options, args) = argprs.parse_known_args(sys.argv[1:])
 
     # Are we debugging this?
     if options.debug:
